<?php

require 'assets/php/models/TimeSlot.php';

$db_path = $_SERVER['DOCUMENT_ROOT'] . '/assets/database.sqlite';

$query_result;
$wanted_slots;

try {
  /*Setup SQL Interface*/
  $pdo = new PDO('sqlite:' . $db_path);
  $pdo->setAttribute(PDO::ATTR_DEFAULT_FETCH_MODE, PDO::FETCH_ASSOC);
  $pdo->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);

  /*Prepare query*/
  $schedule_query = $pdo->prepare("SELECT id, type, start_time, end_time, location
    FROM schedule NATURAL JOIN rooms
    WHERE type LIKE :type AND date(date) = date(:date)
    AND time(end_time) > time(:start_time) AND time(start_time) < time(:end_time)
    ORDER BY start_time");

  /**
   * QUERY DESCRIPTION : 
   * Select all slots mathcing the date & room and where the time slot
   * is overllaping with the requested one. 
   */

  /*Execute query*/
  $schedule_query->execute([
    'type' => $_GET['room_type'],
    'date' => $_GET['date'],
    'start_time' => $_GET['start_time'],
    'end_time' => $_GET['end_time']
  ]);
  $query_result = $schedule_query->fetchAll(PDO::FETCH_CLASS, 'TimeSlot');

  /* Prepare wanted times slots in each room*/
  $room_query = $pdo->prepare("SELECT * FROM rooms WHERE type LIKE :type");
  $room_query->execute(['type' => $_GET['room_type']]);
  $rooms = $room_query->fetchAll(PDO::FETCH_CLASS, 'Room');

  $wanted_slots =
    array_map(
      /*Create new wanted slots for each room*/
      function (Room $r): TimeSlot {
        $slot = new TimeSlot();
        $slot->start_time = $_GET['start_time'];
        $slot->end_time = $_GET['end_time'];
        $slot->room = clone $r;
        return $slot;
      },
      $rooms,
    );

  // Generates results
  $results = [];
  foreach ($wanted_slots as $span) {
    $scissors = array_filter(
      $query_result,
      function (TimeSlot $e) use ($span): bool {
        return $e->room->id === $span->room->id;
      }
    );

    $strips = [$span];
    foreach ($scissors as $blade) {
      $cuts = [];
      foreach ($strips as $sub_span)
        $cuts = array_merge($cuts, $sub_span->cut($blade));

      $strips = $cuts;
    }
    $results = array_merge($results, $strips);
  }
} catch (PDOException $exception) {
  echo var_dump($exception->getMessage());
}

?>
<!DOCTYPE html>
<html lang="fr">

<head>
  <title>UFHOUR - Search</title>

  <!-- Head -->
  <?php require("assets/php/head.php"); ?>
  <link rel="stylesheet" href="assets/css/search.css">
</head>

<body>
  <main>
    <header>
      <h1>UFHOUR</h1>
      <p>Recherche de salles libres de l'UFR Math-Info</p>
    </header>
    <h2>Résultats de la recherche</h2>
    <hr>
    <section>
      <?php
      // Display results
      foreach ($results as $slot)
        $slot->display();
      ?>
    </section>
  </main>
  <?php include "assets/php/cookies.php"; ?>
  <?php include "assets/php/nav.php"; ?>
</body>

</html>